#! Preflu2D 10.4 
#! Flux3D 10.4 
#! FluxSkewed 10.4 

# This macro gives differents values on a 3D curve
# It will allow extracting one specific value of each curve, and creating a new 2D curve showing 
# variation of the average (or min,max,mean, rms, ...) versus the other parameter
# For instance if you compute induced voltage (versus time) on DC generator versus value of current, it will
# allow extracting the curve of rms value versus current.

# date    : August 2011
# version : Flux V10.4.2
# Author  : PL
# update  : April 2013       Add management of 3D curve versus path and parameter
# version : Flux V11.2

import os

"""
@param curve3D      Curve3d                 1 1 None Select a 3D curve (I/O parameter or path)
@param axe          VariationParameterPilot 1 1 None Select a Variation Parameter
@param analyseCurve C80                     1 1 None Write type of analysis: MIN,MAX,MEAN,RECTIFIEDMEAN,RMS,INTEGRAL
"""

def ExtractFrom3DCurve(curve3D,axe,analyseCurve):
   print "\n *** starting macro on extracting values from curve ***\n"

   # Control of right answer for analyseCurve
   list=['MIN','MAX','MEAN','RECTIFIEDMEAN','RMS','INTEGRAL']
   analyseCurve=analyseCurve.upper()
   if list.count(analyseCurve)==0:
      print " ***  Wrong type of analyse, please check the spelling  ***"
      raise RunTimeError()

   #if CurveSpatial3D.existItem(curve3D.name) :
   #   print " ***  3D curve from grid not implemented  ***"
   #   raise RunTimeError()

   # Create table of points for new 2D result curve
   tabXY=[]

   # put batch mode (it will run faster)
   #putBatchMode()

   # Create N 2D curves 
   nameParam1=CurveVariation3D[curve3D.name].creationInformations[-3].strip()
   nameParam2=CurveVariation3D[curve3D.name].creationInformations[-1].strip()
   nameParameter=nameParam1
   xAxis=CurveVariation3D[curve3D.name].xAxis
   xAxis2=CurveVariation3D[curve3D.name].yAxis
   
   if axe.name.split()[0]==nameParam1:
      nameParameter=nameParam2
      xAxis=CurveVariation3D[curve3D.name].yAxis
      xAxis2=CurveVariation3D[curve3D.name].xAxis
      nameParam2=nameParam1
      
   # in case of spatial parameter curve
   list2=CompoundPath[ALL].name
   print "List ",list2, " namePath",nameParameter[2:]
   #try :
   #   condition=list2.count(str(nameParameter[2:]))
   #except:
   #   # no path existing
   #   condition=0
   #print "condition (0= no path)",condition
   #if condition:
   if str(nameParameter[2:]) in list2:
      # in case of spatial parameter curve, remove the U_... from the axis name
      nameParameter=nameParameter[2:]
      nameParam2=nameParam2[2:]

      # find out scenario name
      scenarioName=CurveVariation3D[curve3D.name].creationInformations[0].split()[1]

      # Define the min and max value of parameter
      VMin=float(curve3D.creationInformations[1].split(';')[0].split('[')[1])
      VMax=float(curve3D.creationInformations[1].split(';')[1].split(']')[0])

      k=0
      # set the Flux project on the first step
      Scenario[scenarioName].selectFirstStep()

      # check if interval is fine
      try:
         ComputeCurrentValueParameter(formula=nameParam2,
                                      result='CURRENTVALUEPARAMETER')
      except:
         ScalarResult['CURRENTVALUEPARAMETER'].delete()
         ComputeCurrentValueParameter(formula=nameParam2,
                                      result='CURRENTVALUEPARAMETER')
      ValueParam=ScalarResult['CURRENTVALUEPARAMETER'].value
      if ((ValueParam >= VMin) * (ValueParam<=VMax)) :
         # name of curve to be created
         nameCurve= 'Z_'+nameParam2+'_'+str(k)
         # Check if name of curve already existing
         while CurveSpatial2D.existItem(nameCurve)<>None:
            nameCurve=nameCurve+'_'
            print "name curve modified",nameCurve

         # Create new curve versus path
         displayPathCurve(curve3D, k, nameParam2, nameParameter, nameCurve)

         # Hide the curve
         CurveSpatial2D[nameCurve].HideCurve()

         # Extract the interesting value from the curve (MIN,MAX,MEAN,RMS, ...)
         yvalue=extractValueFromCurve(analyseCurve,CurveSpatial2D[nameCurve])
         # Create one point for the final curve to be created
         tabXY.append(XYCoordinates(xy=[float(ValueParam),yvalue]))

      # loop as long as the next step does exist
      while (     Scenario[scenarioName].existNextStep()==1) :

         # set the Flux project on next step
         Scenario[scenarioName].selectNextStep()

         # check if interval is fine
         try:
            ComputeCurrentValueParameter(formula=nameParam2,
                                         result='CURRENTVALUEPARAMETER')
         except:
            ScalarResult['CURRENTVALUEPARAMETER'].delete()
            ComputeCurrentValueParameter(formula=nameParam2,
                                      result='CURRENTVALUEPARAMETER')
         ValueParam=ScalarResult['CURRENTVALUEPARAMETER'].value

         # increment the index
         k=k+1
         print "\n index k ",k,"\n"


         if ((ValueParam >= VMin) * (ValueParam<=VMax)) :

            # update name of curve to be created
            nameCurve= 'Z_'+nameParam2+'_'+str(k)
            while CurveSpatial2D.existItem(nameCurve)<>None:
               nameCurve=nameCurve+'_'
               print "name curve modified",nameCurve

            # Create new curve versus path
            displayPathCurve(curve3D, k, nameParam2, nameParameter,  nameCurve)


            # Hide the curve
            CurveSpatial2D[nameCurve].HideCurve()

            # Extract the interesting value from the curve (MIN,MAX,MEAN,RMS, ...)
            yvalue=extractValueFromCurve(analyseCurve,CurveSpatial2D[nameCurve])

            # Create one point for the final curve to be created
            tabXY.append(XYCoordinates(xy=[float(ValueParam),yvalue]))



      tabparameterSet="SetParameterXVariable(paramEvol=VariationParameter['"+nameParam2+"'],limitMin="+str(float(VMin))+",limitMax="+str(float(VMax))+")])"

      # Display the new tabulated I/O parameter
      CommandLastCurve="EvolutiveCurve2D(name='"+curve3D.name+"_"+analyseCurve+"', evolutivePath=EvolutivePath(parameterSet=["+tabparameterSet
      CommandLastCurve=CommandLastCurve+",formula=['"+curve3D.name+"_"+analyseCurve+"'])"



# Example of command for creating curve versus parameter (coming from case with path)

#EvolutiveCurve2D(name='EvolutiveCurve2D_2',
#                 evolutivePath=EvolutivePath(parameterSet=[SetParameterXVariable(paramEvol=VariationParameter['TIME'],
#                                                                                 limitMin=6.0,
#                                                                                 limitMax=7.0)]),
#                 formula=['TABULE'])



   else:
      k=0
      # loop for each value of parameter
      for i in xAxis2.x:
         k=k+1
         # Define name of curves
         nameCurve= 'Z_'+axe.name.split()[0]+'_'+str(k)

         # test if curve already existing
         if CurveVariation2D.existItem(nameCurve):
            print "curve already existing"
            CurveVariation2D[nameCurve].delete()
         

         # Define the parameterSet part
         tabparameterSet=""
         for line in CurveVariation3D[curve3D.name].creationInformations:
            #print line
            if "=" in line:
               if "[" in line:
                  continue
               else:
                  var=line.split()
                  #print var
                  tabparameterSet=tabparameterSet+"SetParameterFixed(paramEvol=VariationParameter['"+var[0]+"'],currentValue="+str(float(var[2]))+"),"
                  #print tabparameterSet
         fixedtab=tabparameterSet
         tabparameterSet=tabparameterSet+"SetParameterFixed(paramEvol=VariationParameter['"+axe.name.split()[0]+"'],currentValue="+str(float(i))+"),"
         #print tabparameterSet
         tabparameterSet=tabparameterSet+"SetParameterXVariable(paramEvol=VariationParameter['"+nameParameter+"'],limitMin="+str(xAxis.x[0])+",limitMax="+str(xAxis.x[-1])+")])"
         #print tabparameterSet


         
         # Create I/O parameter curve
         Command="EvolutiveCurve2D(name='"+str(nameCurve)+"', evolutivePath=EvolutivePath(parameterSet=["+tabparameterSet
         Command=Command+",formula=['"+CurveVariation3D[curve3D.name].creationInformations[-5].strip()+"'])"
         print Command
         exec(Command)

         # Hide the curve
         CurveVariation2D[nameCurve].HideCurve()

         # Extract the interesting value from the curve (MIN,MAX,MEAN,RMS, ...)
         yvalue=extractValueFromCurve(analyseCurve,CurveVariation2D[nameCurve])

         # Create one point for the final curve to be created
         tabXY.append(XYCoordinates(xy=[i,yvalue]))


      # Define the parameterSet part
      tabparameterSet=fixedtab+"SetParameterFixed(paramEvol=VariationParameter['"+nameParameter+"'],currentValue="+str(float(xAxis.x[0]))+"),"
      tabparameterSet=tabparameterSet+"SetParameterXVariable(paramEvol=VariationParameter['"+axe.name.split()[0]+"'],limitMin="+str(xAxis2.x[0])+",limitMax="+str(xAxis2.x[-1])+")])"

      # Display the new tabulated I/O parameter
      CommandLastCurve="EvolutiveCurve2D(name='"+curve3D.name+"_"+analyseCurve+"', evolutivePath=EvolutivePath(parameterSet=["+tabparameterSet
      CommandLastCurve=CommandLastCurve+",formula=['"+curve3D.name+"_"+analyseCurve+"'])"



   # Creation of one curve with min, max, of the different created curves   
  
   # example of command for creating a tabulated I/O parameter
   #   lastInstance = VariationParameterTabulated(name='VariationParameter_1',
   #                            xParameter=VariationParameter['BR'],
   #                            curvePoints=[XYCoordinates(xy=[0.0,
   #                                                           2.0]),
   #                                         XYCoordinates(xy=[1.0,
   #                                                           3.0]),
   #                                         XYCoordinates(xy=[2.0,
   #                                                           4.0]),
   #                                         XYCoordinates(xy=[3.0,
   #                                                           5.0])])
   
   # Create a new tabulated I/O parameter
   if VariationParameterTabulated.existItem(curve3D.name+"_"+analyseCurve):
      print "VariationParameterTabulated["+curve3D.name+"_"+analyseCurve+"+ already existing"
      VariationParameterTabulated[curve3D.name+"_"+analyseCurve].delete()
   VariationParameterTabulated(name=curve3D.name+"_"+analyseCurve, xParameter=axe, curvePoints=tabXY)

   # Delete the curve if already existing
   if CurveVariation2D.existItem(curve3D.name+"_"+analyseCurve):
      print "Variation2D["+curve3D.name+"_"+analyseCurve+"] already existing"
      CurveVariation2D[curve3D.name+"_"+analyseCurve].delete()

   # remove batch mode (to display the final curve)
   #resetBatchMode()

   ## Display the new tabulated I/O parameter
   #Command="EvolutiveCurve2D(name='"+curve3D.name+"_"+analyseCurve+"', evolutivePath=EvolutivePath(parameterSet=["+tabparameterSet
   #Command=Command+",formula=['"+curve3D.name+"_"+analyseCurve+"'])"
   print CommandLastCurve
   exec(CommandLastCurve)

   
   # The curve command (except that if there are more than 2 parameters in the scenario, it is more complex!)
   #EvolutiveCurve2D(name=curve3D.name+"_"+analyseCurve,
   #              evolutivePath=EvolutivePath(parameterSet=[SetParameterXVariable(paramEvol=axe,
   #                                                                              limitMin=xAxis2.x[0],
   #                                                                              limitMax=xAxis2.x[-1]),
   #                                                        SetParameterFixed(paramEvol=VariationParameter[nameParameter],
   #                                                                          currentValue=xAxis.x[0])]),
   #              formula=[curve3D.name+"_"+analyseCurve])

   print "\n *** Ending successfully macro on extracting values from curve ***\n"

def extractValueFromCurve(analyseCurve,Curve2D):
   # Function to extract the interesting values from the curve
   print "inside extractValueFromCurve"
   if analyseCurve=='MIN':
      return float(CurveVariation2D[Curve2D.name].y[0].minimalValues[0])
   if analyseCurve=='MAX':
      return float(CurveVariation2D[Curve2D.name].y[0].maximalValues[0])
   if analyseCurve=='MEAN':
      return float(CurveVariation2D[Curve2D.name].y[0].meanValues[0])
   if analyseCurve=='RECTIFIEDMEAN':
      return float(CurveVariation2D[Curve2D.name].y[0].rectifiedMeanValues[0])
   if analyseCurve=='RMS':
      return float(CurveVariation2D[Curve2D.name].y[0].rmsValues[0])
   if analyseCurve=='INTEGRAL':
      return float(CurveVariation2D[Curve2D.name].y[0].integral[0])


def displayPathCurve(curve3D, k, nameParameter, nameParam2,nameCurve):

   Command="SpatialCurve(name='"+str(nameCurve)+"', compoundPath=CompoundPath['"+str(nameParam2)+"']"
   Command=Command+",formula=['"+CurveVariation3D[curve3D.name].creationInformations[-5].strip()+"'])"
         
   print Command
   try:
      exec(Command)
   except:
      print "curve already existing, to be deleted",nameCurve
      CurveSpatial2D[nameCurve].delete()
      exec(Command)


